/* Application Explorer - a general-purpose application cataloger/indexer
 * Copyright (C) 2006  Aliasgar Lokhandwala <d7@freepgs.com> 
 * http://appexp.sourceforge.net/
 *
 * BrwFile.java: the reader for .brw type application descriptors
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package app.appexplorer.file;

import java.io.*;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;

/**
 * 
 * This class provides support for reading .brw files.
 * 
 * The BrwFile is the main and simplest "application descriptor" file. NOTE: A
 * brw file is an ini file.
 * 
 * @author ali
 * @version 2.0
 */
public final class BrwFile extends IniFile implements IBrwFile {

    private String m_sCategory = "Default";

    private String m_sName = "UnknownApplication";

    private String m_sShortDesc = null;

    private String m_sLongDesc = null;

    private Vector m_vtSetupFiles = null;

    private boolean m_bSearchSubDirs = false;

    private static Logger m_globalLogger = LogManager.getLogManager()
            .getLogger("global");

    /**
     * Constructs a Brw File object. NOTE: Access this object using the IBrwFile
     * interface.
     * 
     * @see IBrwFile
     * @param file -
     *            a File object pointing to the source .brw file on disk. Should
     *            not be null.
     */
    public BrwFile(File file) {
        super(file);
        //read the information section
        Hashtable h = readSection("info");

        //populate category and name fields from the section
        String value = (String) h.get("category");
        if (value != null) {
            m_sCategory = value;
        }

        value = (String) h.get("name");
        if (value != null) {
            m_sName = value;
        }
    }

    /*
     * Return the Category of the application as read from the [info] section.
     * @return String
     */
    public String getCategory() {
        return m_sCategory;
    }

    /*
     * Return the Name of the application as read from the [info] section.
     * @return String
     */
    public String getName() {
        return m_sName;
    }

    /**
     * This override causes the name of the application to be shown when this
     * object is added as a leaf node in the JTree.
     * 
     * @return String
     * @see javax.swing.JTree
     */
    public String toString() {
        return m_sName;
    }

    /*
     * Return the path of this file on disk.
     */
    public String getPath() {
        return m_file.getAbsolutePath();
    }

    /*
     * Returns the folder containing this file.
     */
    public String getContainingFolder() {
        return m_file.getParent();
    }

    /**
     * Read the text file given by sFileName as the description.
     * 
     * @param sFileName
     * @return The contents of the file given by sFileName or null if fails.
     */
    private String readDescFile(String sFileName) {

        m_globalLogger.log(Level.FINE, "Will read description from file "
                + sFileName);

        File file = new File(sFileName);
        String desc = "";
        BufferedReader inBuffer = null;
        try {
            if (file.canRead()) {
                //go ahead and read all lines into desc.
                inBuffer = new BufferedReader(new FileReader(file));
                String line = null;
                while ((line = inBuffer.readLine()) != null) {
                    desc += (line + "\n");
                }
                inBuffer.close();
            } else {
                //we dont have read permission on the file
                desc = null;
                m_globalLogger.log(Level.WARNING,
                        "Description file could not be read.");
            }
        } catch (IOException e) {
            desc = null;
            m_globalLogger.log(Level.SEVERE, "IO Exception reading file "
                    + e.getMessage());
        }

        return desc;
    }

    /**
     * Reads options given in the [showfiles] section.
     */
    private void readSetupFileOptions() {
        /*
         * Read the [showfiles] section. The hashtable consists of a list of
         * string name=value pairs filters {"file1"= "*.msi", "file2"="*.txt"}.
         * 
         * Additionally it can contain a string key "searchsubdirs" with value
         * "true" or "false"
         */
        Hashtable htb = readSection("showfiles");

        //See if we need to search subfolders
        String searchsub = (String) htb.get("searchsubdirs");

        if ("true".equalsIgnoreCase(searchsub)
                || "yes".equalsIgnoreCase(searchsub)) {
            m_bSearchSubDirs = true;
            m_globalLogger.log(Level.FINE,
                    "Will search subfolders for setup files.");
        }

        //read the filters
        m_vtSetupFiles = new Vector();

        //get the list of filters
        int i = 1;
        while (htb.get("file" + i) != null) {
            m_vtSetupFiles.add(htb.get("file" + i));
            i++;
        }

        m_globalLogger.log(Level.FINE, "Found " + (i - 1)
                + "Setup file filters");
    }

    /**
     * Gets the brief and long descriptions for this app.
     */
    private void getDescriptionTexts() {
        Hashtable h = readSection("description");

        String line = null;

        /*
         * The description can either be given directly in the brw file using
         * lineN=value pairs (short description) or may be given by pointing to
         * another text file using the file=descFile.txt option (long
         * desctiption).
         */

        //read the long description
        if ((line = (String) h.get("file")) != null) {
            //return the description by reading the external file
            m_sLongDesc = readDescFile(getContainingFolder() + "\\" + line);
        }

        //read the short description from the brw file itself
        String desc = "";
        int i = 1;
        while ((line = (String) h.get("line" + i)) != null) {
            desc += (line + "\n");
            i++;
        }
        if (desc.length() > 0) {
            m_sShortDesc = desc;
        }

        //if we couldnt find either
        if (m_sLongDesc == null && m_sShortDesc == null) {
            m_sShortDesc = "No description available.";
        }

        //we found the short description, but not the long one
        //so make the long description same as short
        if (m_sLongDesc == null && m_sShortDesc != null) {
            m_sLongDesc = m_sShortDesc;
        }

        //we found the long description, but not the short one
        if (m_sLongDesc != null && m_sShortDesc == null) {
            m_sShortDesc = m_sLongDesc;
        }
    }

    /*
     * Returns the short description read from the file
     * 
     * @see app.appexplorer.file.IBrwFile#getShortDescriptionText()
     */
    public String getShortDescriptionText() {
        if (m_sShortDesc == null)
            getDescriptionTexts();

        return m_sShortDesc;
    }

    /*
     * Returns the long description read from the file
     * 
     * @see app.appexplorer.file.IBrwFile#getLongDescriptionText()
     */
    public String getLongDescriptionText() {
        if (m_sLongDesc == null)
            getDescriptionTexts();

        return m_sLongDesc;
    }

    /*
     * Gets a vector containing filters used to search for setup files.
     * 
     * @see app.appexplorer.file.IBrwFile#getSetupFileFilters()
     */
    public Vector getSetupFileFilters() {
        //if this is the first time this info is requested, read it from file
        if (m_vtSetupFiles == null)
            readSetupFileOptions();

        return m_vtSetupFiles;
    }

    /*
     * @see app.appexplorer.file.IBrwFile#SearchSubFoldersForSetupFiles()
     */
    public boolean searchSubFoldersForSetupFiles() {
        //if this is the first time this info is requested, read it from file
        if (m_vtSetupFiles == null)
            readSetupFileOptions();

        return m_bSearchSubDirs;
    }
}